@@ -1,6 +1,6 @@
 /* Allocate registers within a basic block, for GNU compiler.
    Copyright (C) 1987, 1988, 1991, 1993, 1994, 1995, 1996, 1997, 1998,
-   1999, 2000, 2001, 2002, 2003, 2004 Free Software Foundation, Inc.
+   1999, 2000, 2001, 2002, 2003, 2004, 2005 Free Software Foundation, Inc.
 
 This file is part of GCC.
 
@@ -16,8 +16,8 @@ for more details.
 
 You should have received a copy of the GNU General Public License
 along with GCC; see the file COPYING.  If not, write to the Free
-Software Foundation, 59 Temple Place - Suite 330, Boston, MA
-02111-1307, USA.  */
+Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301, USA.  */
 
 /* Allocation of hard register numbers to pseudo registers is done in
    two passes.  In this pass we consider only regs that are born and
@@ -76,6 +76,10 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 #include "toplev.h"
 #include "except.h"
 #include "integrate.h"
+#include "reload.h"
+#include "ggc.h"
+#include "timevar.h"
+#include "tree-pass.h"
 
 /* Next quantity number available for allocation.  */
 
@@ -265,6 +269,10 @@ struct equivalence
   /* The list of each instruction which initializes this register.  */
 
   rtx init_insns;
+
+  /* Nonzero if this had a preexisting REG_EQUIV note.  */
+
+  int is_arg_equivalence;
 };
 
 /* reg_equiv[N] (where N is a pseudo reg number) is the equivalence
@@ -350,8 +358,7 @@ local_alloc (void)
 
   /* Promote REG_EQUAL notes to REG_EQUIV notes and adjust status of affected
      registers.  */
-  if (optimize)
-    update_equiv_regs ();
+  update_equiv_regs ();
 
   /* This sets the maximum number of quantities we can have.  Quantity
      numbers start at zero and we can have one for each pseudo.  */
@@ -755,8 +762,19 @@ memref_used_between_p (rtx memref, rtx start, rtx end)
 
   for (insn = NEXT_INSN (start); insn != NEXT_INSN (end);
        insn = NEXT_INSN (insn))
-    if (INSN_P (insn) && memref_referenced_p (memref, PATTERN (insn)))
-      return 1;
+    {
+      if (!INSN_P (insn))
+	continue;
+      
+      if (memref_referenced_p (memref, PATTERN (insn)))
+	return 1;
+
+      /* Nonconst functions may access memory.  */
+      if (CALL_P (insn)
+	  && (! CONST_OR_PURE_CALL_P (insn)
+	      || pure_call_p (insn)))
+	return 1;
+    }
 
   return 0;
 }
@@ -767,7 +785,9 @@ memref_used_between_p (rtx memref, rtx start, rtx end)
 
    If such a register is only referenced once, try substituting its value
    into the using insn.  If it succeeds, we can eliminate the register
-   completely.  */
+   completely.
+
+   Initialize the REG_EQUIV_INIT array of initializing insns.  */
 
 static void
 update_equiv_regs (void)
@@ -780,6 +800,8 @@ update_equiv_regs (void)
 
   reg_equiv = xcalloc (max_regno, sizeof *reg_equiv);
   INIT_REG_SET (&cleared_regs);
+  reg_equiv_init = ggc_alloc_cleared (max_regno * sizeof (rtx));
+  reg_equiv_init_size = max_regno;
 
   init_alias_analysis ();
 
@@ -830,44 +852,30 @@ update_equiv_regs (void)
 	  dest = SET_DEST (set);
 	  src = SET_SRC (set);
 
-	  /* If this sets a MEM to the contents of a REG that is only used
-	     in a single basic block, see if the register is always equivalent
-	     to that memory location and if moving the store from INSN to the
-	     insn that set REG is safe.  If so, put a REG_EQUIV note on the
-	     initializing insn.
-
-	     Don't add a REG_EQUIV note if the insn already has one.  The existing
-	     REG_EQUIV is likely more useful than the one we are adding.
-
-	     If one of the regs in the address has reg_equiv[REGNO].replace set,
-	     then we can't add this REG_EQUIV note.  The reg_equiv[REGNO].replace
-	     optimization may move the set of this register immediately before
-	     insn, which puts it after reg_equiv[REGNO].init_insns, and hence
-	     the mention in the REG_EQUIV note would be to an uninitialized
-	     pseudo.  */
-	  /* ????? This test isn't good enough; we might see a MEM with a use of
-	     a pseudo register before we see its setting insn that will cause
-	     reg_equiv[].replace for that pseudo to be set.
-	     Equivalences to MEMs should be made in another pass, after the
-	     reg_equiv[].replace information has been gathered.  */
-
-	  if (MEM_P (dest) && REG_P (src)
-	      && (regno = REGNO (src)) >= FIRST_PSEUDO_REGISTER
-	      && REG_BASIC_BLOCK (regno) >= 0
-	      && REG_N_SETS (regno) == 1
-	      && reg_equiv[regno].init_insns != 0
-	      && reg_equiv[regno].init_insns != const0_rtx
-	      && ! find_reg_note (XEXP (reg_equiv[regno].init_insns, 0),
-				  REG_EQUIV, NULL_RTX)
-	      && ! contains_replace_regs (XEXP (dest, 0)))
+	  /* See if this is setting up the equivalence between an argument
+	     register and its stack slot.  */
+	  note = find_reg_note (insn, REG_EQUIV, NULL_RTX);
+	  if (note)
 	    {
-	      rtx init_insn = XEXP (reg_equiv[regno].init_insns, 0);
-	      if (validate_equiv_mem (init_insn, src, dest)
-		  && ! memref_used_between_p (dest, init_insn, insn))
-		REG_NOTES (init_insn)
-		  = gen_rtx_EXPR_LIST (REG_EQUIV, dest, REG_NOTES (init_insn));
+	      gcc_assert (REG_P (dest));
+	      regno = REGNO (dest);
+
+	      /* Note that we don't want to clear reg_equiv_init even if there
+		 are multiple sets of this register.  */
+	      reg_equiv[regno].is_arg_equivalence = 1;
+
+	      /* Record for reload that this is an equivalencing insn.  */
+	      if (rtx_equal_p (src, XEXP (note, 0)))
+		reg_equiv_init[regno]
+		  = gen_rtx_INSN_LIST (VOIDmode, insn, reg_equiv_init[regno]);
+
+	      /* Continue normally in case this is a candidate for
+		 replacements.  */
 	    }
 
+	  if (!optimize)
+	    continue;
+
 	  /* We only handle the case of a pseudo register being set
 	     once, or always to the same value.  */
 	  /* ??? The mn10200 port breaks if we add equivalences for
@@ -884,7 +892,7 @@ update_equiv_regs (void)
 	      || (regno = REGNO (dest)) < FIRST_PSEUDO_REGISTER
 	      || reg_equiv[regno].init_insns == const0_rtx
 	      || (CLASS_LIKELY_SPILLED_P (reg_preferred_class (regno))
-		  && MEM_P (src)))
+		  && MEM_P (src) && ! reg_equiv[regno].is_arg_equivalence))
 	    {
 	      /* This might be setting a SUBREG of a pseudo, a pseudo that is
 		 also set somewhere else to a constant.  */
@@ -950,20 +958,26 @@ update_equiv_regs (void)
 	  if (note)
 	    {
 	      int regno = REGNO (dest);
+	      rtx x = XEXP (note, 0);
+
+	      /* If we haven't done so, record for reload that this is an
+		 equivalencing insn.  */
+	      if (!reg_equiv[regno].is_arg_equivalence)
+		reg_equiv_init[regno]
+		  = gen_rtx_INSN_LIST (VOIDmode, insn, reg_equiv_init[regno]);
 
 	      /* Record whether or not we created a REG_EQUIV note for a LABEL_REF.
 		 We might end up substituting the LABEL_REF for uses of the
 		 pseudo here or later.  That kind of transformation may turn an
 		 indirect jump into a direct jump, in which case we must rerun the
 		 jump optimizer to ensure that the JUMP_LABEL fields are valid.  */
-	      if (GET_CODE (XEXP (note, 0)) == LABEL_REF
-		  || (GET_CODE (XEXP (note, 0)) == CONST
-		      && GET_CODE (XEXP (XEXP (note, 0), 0)) == PLUS
-		      && (GET_CODE (XEXP (XEXP (XEXP (note, 0), 0), 0))
-			  == LABEL_REF)))
+	      if (GET_CODE (x) == LABEL_REF
+		  || (GET_CODE (x) == CONST
+		      && GET_CODE (XEXP (x, 0)) == PLUS
+		      && (GET_CODE (XEXP (XEXP (x, 0), 0)) == LABEL_REF)))
 		recorded_label_ref = 1;
 
-	      reg_equiv[regno].replacement = XEXP (note, 0);
+	      reg_equiv[regno].replacement = x;
 	      reg_equiv[regno].src_p = &SET_SRC (set);
 	      reg_equiv[regno].loop_depth = loop_depth;
 
@@ -974,7 +988,6 @@ update_equiv_regs (void)
 		     in local-alloc!  */
 		  REG_LIVE_LENGTH (regno) *= 2;
 
-
 		  /* If the register is referenced exactly twice, meaning it is
 		     set once and used once, indicate that the reference may be
 		     replaced by the equivalence we computed above.  Do this
@@ -984,20 +997,82 @@ update_equiv_regs (void)
 		     and to reduce the number of registers alive across
 		     calls.  */
 
-		    if (REG_N_REFS (regno) == 2
-			&& (rtx_equal_p (XEXP (note, 0), src)
-			    || ! equiv_init_varies_p (src))
-			&& NONJUMP_INSN_P (insn)
-			&& equiv_init_movable_p (PATTERN (insn), regno))
-		      reg_equiv[regno].replace = 1;
+		  if (REG_N_REFS (regno) == 2
+		      && (rtx_equal_p (x, src)
+			  || ! equiv_init_varies_p (src))
+		      && NONJUMP_INSN_P (insn)
+		      && equiv_init_movable_p (PATTERN (insn), regno))
+		    reg_equiv[regno].replace = 1;
 		}
 	    }
 	}
     }
 
+  if (!optimize)
+    goto out;
+
+  /* A second pass, to gather additional equivalences with memory.  This needs
+     to be done after we know which registers we are going to replace.  */
+
+  for (insn = get_insns (); insn; insn = NEXT_INSN (insn))
+    {
+      rtx set, src, dest;
+      unsigned regno;
+
+      if (! INSN_P (insn))
+	continue;
+
+      set = single_set (insn);
+      if (! set)
+	continue;
+
+      dest = SET_DEST (set);
+      src = SET_SRC (set);
+
+      /* If this sets a MEM to the contents of a REG that is only used
+	 in a single basic block, see if the register is always equivalent
+	 to that memory location and if moving the store from INSN to the
+	 insn that set REG is safe.  If so, put a REG_EQUIV note on the
+	 initializing insn.
+
+	 Don't add a REG_EQUIV note if the insn already has one.  The existing
+	 REG_EQUIV is likely more useful than the one we are adding.
+
+	 If one of the regs in the address has reg_equiv[REGNO].replace set,
+	 then we can't add this REG_EQUIV note.  The reg_equiv[REGNO].replace
+	 optimization may move the set of this register immediately before
+	 insn, which puts it after reg_equiv[REGNO].init_insns, and hence
+	 the mention in the REG_EQUIV note would be to an uninitialized
+	 pseudo.  */
+
+      if (MEM_P (dest) && REG_P (src)
+	  && (regno = REGNO (src)) >= FIRST_PSEUDO_REGISTER
+	  && REG_BASIC_BLOCK (regno) >= 0
+	  && REG_N_SETS (regno) == 1
+	  && reg_equiv[regno].init_insns != 0
+	  && reg_equiv[regno].init_insns != const0_rtx
+	  && ! find_reg_note (XEXP (reg_equiv[regno].init_insns, 0),
+			      REG_EQUIV, NULL_RTX)
+	  && ! contains_replace_regs (XEXP (dest, 0)))
+	{
+	  rtx init_insn = XEXP (reg_equiv[regno].init_insns, 0);
+	  if (validate_equiv_mem (init_insn, src, dest)
+	      && ! memref_used_between_p (dest, init_insn, insn))
+	    {
+	      REG_NOTES (init_insn)
+		= gen_rtx_EXPR_LIST (REG_EQUIV, dest,
+				     REG_NOTES (init_insn));
+	      /* This insn makes the equivalence, not the one initializing
+		 the register.  */
+	      reg_equiv_init[regno]
+		= gen_rtx_INSN_LIST (VOIDmode, insn, NULL_RTX);
+	    }
+	}
+    }
+
   /* Now scan all regs killed in an insn to see if any of them are
      registers only used that once.  If so, see if we can replace the
-     reference with the equivalent from.  If we can, delete the
+     reference with the equivalent form.  If we can, delete the
      initializing reference and this register will go away.  If we
      can't replace the reference, and the initializing reference is
      within the same loop (or in an inner loop), then move the register
@@ -1038,9 +1113,8 @@ update_equiv_regs (void)
 		     once and used once.  (If it were only set, but not used,
 		     flow would have deleted the setting insns.)  Hence
 		     there can only be one insn in reg_equiv[REGNO].init_insns.  */
-		  if (reg_equiv[regno].init_insns == NULL_RTX
-		      || XEXP (reg_equiv[regno].init_insns, 1) != NULL_RTX)
-		    abort ();
+		  gcc_assert (reg_equiv[regno].init_insns
+			      && !XEXP (reg_equiv[regno].init_insns, 1));
 		  equiv_insn = XEXP (reg_equiv[regno].init_insns, 0);
 
 		  /* We may not move instructions that can throw, since
@@ -1084,6 +1158,12 @@ update_equiv_regs (void)
 
 		      reg_equiv[regno].init_insns
 			= XEXP (reg_equiv[regno].init_insns, 1);
+
+		      /* Remember to clear REGNO from all basic block's live
+			 info.  */
+		      SET_REGNO_REG_SET (&cleared_regs, regno);
+		      clear_regnos++;
+		      reg_equiv_init[regno] = NULL_RTX;
 		    }
 		  /* Move the initialization of the register to just before
 		     INSN.  Update the flow information.  */
@@ -1095,8 +1175,9 @@ update_equiv_regs (void)
 		      REG_NOTES (new_insn) = REG_NOTES (equiv_insn);
 		      REG_NOTES (equiv_insn) = 0;
 
-		      /* Make sure this insn is recognized before reload begins,
-			 otherwise eliminate_regs_in_insn will abort.  */
+		      /* Make sure this insn is recognized before
+			 reload begins, otherwise
+			 eliminate_regs_in_insn will die.  */
 		      INSN_CODE (new_insn) = INSN_CODE (equiv_insn);
 
 		      delete_insn (equiv_insn);
@@ -1115,6 +1196,8 @@ update_equiv_regs (void)
 			 info.  */
 		      SET_REGNO_REG_SET (&cleared_regs, regno);
 		      clear_regnos++;
+		      reg_equiv_init[regno]
+			= gen_rtx_INSN_LIST (VOIDmode, new_insn, NULL_RTX);
 		    }
 		}
 	    }
@@ -1130,8 +1213,10 @@ update_equiv_regs (void)
 	{
 	  FOR_EACH_BB (bb)
 	    {
-	      AND_COMPL_REG_SET (bb->global_live_at_start, &cleared_regs);
-	      AND_COMPL_REG_SET (bb->global_live_at_end, &cleared_regs);
+	      AND_COMPL_REG_SET (bb->il.rtl->global_live_at_start,
+			         &cleared_regs);
+	      AND_COMPL_REG_SET (bb->il.rtl->global_live_at_end,
+			         &cleared_regs);
 	    }
 	}
       else
@@ -1141,13 +1226,14 @@ update_equiv_regs (void)
 	    {
 	      FOR_EACH_BB (bb)
 		{
-		  CLEAR_REGNO_REG_SET (bb->global_live_at_start, j);
-		  CLEAR_REGNO_REG_SET (bb->global_live_at_end, j);
+		  CLEAR_REGNO_REG_SET (bb->il.rtl->global_live_at_start, j);
+		  CLEAR_REGNO_REG_SET (bb->il.rtl->global_live_at_end, j);
 		}
 	    }
 	}
     }
 
+  out:
   /* Clean up.  */
   end_alias_analysis ();
   CLEAR_REG_SET (&cleared_regs);
@@ -1173,13 +1259,18 @@ no_equiv (rtx reg, rtx store ATTRIBUTE_UNUSED, void *data ATTRIBUTE_UNUSED)
   list = reg_equiv[regno].init_insns;
   if (list == const0_rtx)
     return;
+  reg_equiv[regno].init_insns = const0_rtx;
+  reg_equiv[regno].replacement = NULL_RTX;
+  /* This doesn't matter for equivalences made for argument registers, we
+     should keep their initialization insns.  */
+  if (reg_equiv[regno].is_arg_equivalence)
+    return;
+  reg_equiv_init[regno] = NULL_RTX;
   for (; list; list =  XEXP (list, 1))
     {
       rtx insn = XEXP (list, 0);
       remove_note (insn, find_reg_note (insn, REG_EQUIV, NULL_RTX));
     }
-  reg_equiv[regno].init_insns = const0_rtx;
-  reg_equiv[regno].replacement = NULL_RTX;
 }
 
 /* Allocate hard regs to the pseudo regs used only within block number B.
@@ -1203,8 +1294,10 @@ block_alloc (int b)
   while (1)
     {
       if (!NOTE_P (insn))
-	if (++insn_count > max_uid)
-	  abort ();
+	{
+	  ++insn_count;
+	  gcc_assert (insn_count <= max_uid);
+	}
       if (insn == BB_HEAD (BASIC_BLOCK (b)))
 	break;
       insn = PREV_INSN (insn);
@@ -1216,7 +1309,8 @@ block_alloc (int b)
 
   /* Initialize table of hardware registers currently live.  */
 
-  REG_SET_TO_HARD_REG_SET (regs_live, BASIC_BLOCK (b)->global_live_at_start);
+  REG_SET_TO_HARD_REG_SET (regs_live,
+		  	   BASIC_BLOCK (b)->il.rtl->global_live_at_start);
 
   /* This loop scans the instructions of the basic block
      and assigns quantities to registers.
@@ -2126,8 +2220,7 @@ find_free_reg (enum reg_class class, enum machine_mode mode, int qtyno,
 #endif
 
   /* Validate our parameters.  */
-  if (born_index < 0 || born_index > dead_index)
-    abort ();
+  gcc_assert (born_index >= 0 && born_index <= dead_index);
 
   /* Don't let a pseudo live in a reg across a function call
      if we might get a nonlocal goto.  */
@@ -2426,3 +2519,69 @@ dump_local_alloc (FILE *file)
     if (reg_renumber[i] != -1)
       fprintf (file, ";; Register %d in %d.\n", i, reg_renumber[i]);
 }
+
+/* Run old register allocator.  Return TRUE if we must exit
+   rest_of_compilation upon return.  */
+static void
+rest_of_handle_local_alloc (void)
+{
+  int rebuild_notes;
+
+  /* Determine if the current function is a leaf before running reload
+     since this can impact optimizations done by the prologue and
+     epilogue thus changing register elimination offsets.  */
+  current_function_is_leaf = leaf_function_p ();
+
+  /* Allocate the reg_renumber array.  */
+  allocate_reg_info (max_regno, FALSE, TRUE);
+
+  /* And the reg_equiv_memory_loc array.  */
+  VARRAY_GROW (reg_equiv_memory_loc_varray, max_regno);
+  reg_equiv_memory_loc = &VARRAY_RTX (reg_equiv_memory_loc_varray, 0);
+
+  allocate_initial_values (reg_equiv_memory_loc);
+
+  regclass (get_insns (), max_reg_num (), dump_file);
+  rebuild_notes = local_alloc ();
+
+  /* Local allocation may have turned an indirect jump into a direct
+     jump.  If so, we must rebuild the JUMP_LABEL fields of jumping
+     instructions.  */
+  if (rebuild_notes)
+    {
+      timevar_push (TV_JUMP);
+
+      rebuild_jump_labels (get_insns ());
+      purge_all_dead_edges ();
+      delete_unreachable_blocks ();
+
+      timevar_pop (TV_JUMP);
+    }
+
+  if (dump_enabled_p (pass_local_alloc.static_pass_number))
+    {
+      timevar_push (TV_DUMP);
+      dump_flow_info (dump_file);
+      dump_local_alloc (dump_file);
+      timevar_pop (TV_DUMP);
+    }
+}
+
+struct tree_opt_pass pass_local_alloc =
+{
+  "lreg",                               /* name */
+  NULL,                                 /* gate */
+  rest_of_handle_local_alloc,           /* execute */
+  NULL,                                 /* sub */
+  NULL,                                 /* next */
+  0,                                    /* static_pass_number */
+  TV_LOCAL_ALLOC,                       /* tv_id */
+  0,                                    /* properties_required */
+  0,                                    /* properties_provided */
+  0,                                    /* properties_destroyed */
+  0,                                    /* todo_flags_start */
+  TODO_dump_func |
+  TODO_ggc_collect,                     /* todo_flags_finish */
+  'l'                                   /* letter */
+};
+
